declare module "utils" {
    /**
     * @private
     */
    export function flatten(object: {
        [key: string]: any;
    }, separator?: string): {
        [key: string]: any;
    };
    /**
     * @private
     */
    export function mergeRoles(dst: {
        [key: string]: any;
    }, ...srcs: Array<{
        [key: string]: any;
    }>): {
        [key: string]: any;
    };
}
declare module "index" {
    /**
     * RBAC classref
     */
    class RBAC {
        /**
         * RBAC roles object
         */
        private _rules;
        private _rulesCompiled;
        private _refs;
        private _memoize;
        private _collectRefs;
        private _compile;
        /**
         * RBAC constructor
         * @param options RBAC options
         */
        constructor({ roles, memoize }?: RBAC.Options);
        /**
         * Adds new role to rules.
         * @public
         * @version 1.1.X
         * @param role user role
         * @param resource resource to access
         * @param operation allowed operation
         * @param when function for additional checks
         */
        add(role: string, resource: string, operation: string, when?: RBAC.WhenFn): void;
        /**
         * Remove rule(s).
         * @public
         * @version 1.1.X
         * @param role user role
         * @param resource resource to access
         * @param operation operation
         */
        remove(role: string, resource?: string, operation?: string): void;
        /**
         * Checks if user can perform operation without checking when condition.
         * @public
         * @version 1.X.X
         * @param role user role
         * @param resource resource to access
         * @param operation operation on resource
         * @returns true if role has access to resources
         */
        can(role: string, resource: string, operation?: string): boolean;
        /**
         * Checks if user can perform operation with checking when condition if it's provided.
         * @public
         * @version 1.X.X
         * @param role user role
         * @param resource resource to access
         * @param operation operation on resource
         * @param context context passed to when function, set it to null
         * @returns true if role has access to resources.
         */
        can(role: string, resource: string, operation: string, context: any): Promise<boolean>;
    }
    namespace RBAC {
        type WhenFn = (context: any) => boolean | Promise<boolean>;
        interface Refs {
            [roleName: string]: RoleRules;
        }
        interface OperationRules {
            [operationName: string]: boolean | WhenFn;
        }
        interface ResourceRules {
            [resourceName: string]: OperationRules;
        }
        interface RoleRules {
            [roleName: string]: ResourceRules;
        }
        interface ResourcePermission {
            name: string;
            operation?: string;
            when?: WhenFn;
        }
        interface RulesObject {
            /**
             * List of resource and permissions.
             * @example
             * can: ["foo:create", "bar:*", "*:read"]
             * can: ["*"]
             * can: [{
             *   name: "baz",
             *   operation: "create",
             *   when: (ctx) => {
             *     return ctx.user.id === ctx.obj.creatorId
             *   }
             * }]
             *
             */
            can: Array<string | ResourcePermission>;
            /**
             * Optionally extend permissions from other roles.
             */
            inherits?: Array<string>;
        }
        interface Options {
            /**
             * Initial roles with permissions.
             */
            roles?: {
                [roleName: string]: RBAC.RulesObject;
            };
            /**
             * If true makes wildcard matches faster
             * @default true
             */
            memoize?: boolean;
        }
    }
    export default RBAC;
}
//# sourceMappingURL=rbac.bundle.amd.d.ts.map